"use strict";

var _react = _interopRequireDefault(require("react"));

var _chai = require("chai");

var _react2 = require("@testing-library/react");

var _Draggable = require("./Draggable");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

describe('draggable', () => {
  const defaultStyle = {
    position: 'fixed',
    top: '11px',
    left: '11px'
  };
  let utils;
  beforeEach(() => {
    utils = setup();
  });
  afterEach(_react2.cleanup);
  it('should supply proper props to target', () => {
    (0, _chai.expect)(utils.container.querySelector('[aria-grabbed]')).not.to.be.ok;
  });
  describe('starting drag', () => {
    it('should supply proper props to target', () => {
      const {
        getByText,
        container
      } = utils;

      _react2.fireEvent.mouseDown(getByText(/handle/));

      (0, _chai.expect)(container.querySelector('[aria-grabbed]')).to.be.ok;
    });
    it('should return a delta position', () => {
      const startAt = {
        clientX: 10,
        clientY: 10
      };
      const delta = {
        x: 5,
        y: 5
      };
      const {
        drag,
        getByText
      } = utils;
      drag({
        start: startAt,
        delta
      });
      (0, _chai.expect)(getByText(`${delta.x}, ${delta.y}`)).to.be.ok;
    });
    it('should return a correct delta position after more drag', () => {
      const startAt = {
        clientX: 10,
        clientY: 10
      };
      const delta = {
        x: 5,
        y: 5
      };
      const secondStart = {
        clientX: startAt.clientX + delta.x,
        clientY: startAt.clientY + delta.y
      };
      const {
        drag,
        getByText
      } = utils;
      drag({
        start: startAt,
        delta
      });
      drag({
        start: secondStart,
        delta
      });
      (0, _chai.expect)(getByText(`${2 * delta.x}, ${2 * delta.y}`)).to.be.ok;
    });
    describe('decorate with styles', () => {
      beforeEach(() => {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true
        });
      });
      it("should change target's style when dragging", () => {
        const {
          getByTestId,
          drag
        } = utils;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          }
        });
        (0, _chai.expect)(getByTestId('main').style.transform).to.equal(`translate(10px, 15px)`);
      });
      it("should change target's style when dragging (touches)", () => {
        const {
          getByTestId,
          drag
        } = utils;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          },
          touch: true
        });
        (0, _chai.expect)(getByTestId('main').style.transform).to.equal(`translate(10px, 15px)`);
      });
      it('should set proper cursor', () => {
        (0, _chai.expect)(utils.getByText(/handle/i).style.cursor).to.equal('grab');
      });
      it('should change cursor while dragging', () => {
        const {
          getByText
        } = utils;

        _react2.fireEvent.mouseDown(getByText(/handle/i));

        (0, _chai.expect)(getByText(/handle/i).style.cursor).to.equal('grabbing');
      });
      it('should add `will-change: transform` to target', () => {
        const {
          getByText,
          getByTestId
        } = utils;

        _react2.fireEvent.mouseDown(getByText(/handle/));

        (0, _chai.expect)(getByTestId('main').style.willChange).to.equal('transform');
      });
      it('should remove `will-change: transform` from target', () => {
        const {
          getByTestId,
          drag
        } = utils;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          }
        });
        (0, _chai.expect)(getByTestId('main').style.willChange).to.equal('');
      });
    });
    describe('ending drag', () => {
      it('should supply proper props to target', () => {
        const {
          drag,
          container
        } = utils;
        drag();
        (0, _chai.expect)(container.querySelector('[aria-grabbed]')).not.to.be.ok;
      });
    });
    describe('limit in viewport', () => {
      beforeEach(() => {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true,
          viewport: true,
          style: { ...defaultStyle,
            width: '180px',
            left: 'auto',
            right: '0'
          }
        });
      });
      it('should not change transition beyond given rect', () => {
        const {
          drag,
          getByTestId
        } = utils;
        const targetElement = getByTestId('main');
        const rect = targetElement.getBoundingClientRect();
        const startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        const delta = {
          x: 5,
          y: 5
        };
        drag({
          start: startAt,
          delta
        });
        (0, _chai.expect)(targetElement.style.transform).to.include(`translate(0px, 5px)`);
      });
      it('should leave transition as it was before limit', () => {
        const {
          getByTestId,
          beginDrag,
          move
        } = utils;
        const targetElement = getByTestId('main');
        targetElement.style.right = '50px';
        const rect = targetElement.getBoundingClientRect();
        const startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        const delta = {
          x: 5,
          y: 1
        };
        beginDrag(startAt);
        move({
          clientX: startAt.clientX + delta.x,
          clientY: startAt.clientY + delta.y
        });
        move({
          clientX: startAt.clientX + delta.x + 10,
          clientY: startAt.clientY + delta.y
        });
        move({
          clientX: startAt.clientX + delta.x + 25,
          clientY: startAt.clientY + delta.y
        });
        move({
          clientX: startAt.clientX + delta.x + 50,
          clientY: startAt.clientY + delta.y
        });
        const {
          left,
          width
        } = targetElement.getBoundingClientRect();
        (0, _chai.expect)(left).to.equal(window.innerWidth - width);
      });
      it('should keep limits when dragging more than once', () => {
        const {
          drag,
          getByTestId
        } = utils;
        const targetElement = getByTestId('main');
        targetElement.style.right = '50px';
        const rect = targetElement.getBoundingClientRect();
        const startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        const delta = {
          x: 15,
          y: 1
        };
        drag({
          start: startAt,
          delta
        });
        drag({
          start: {
            clientX: startAt.clientX + delta.x,
            clientY: startAt.clientY + delta.y
          },
          delta: {
            x: 50,
            y: 0
          }
        });
        const {
          left,
          width
        } = targetElement.getBoundingClientRect();
        (0, _chai.expect)(left).to.equal(window.innerWidth - width);
      });
    });
    describe('limit in rect', () => {
      const limits = {
        left: 11,
        right: window.innerWidth - 11,
        top: 5,
        bottom: window.innerHeight - 13
      };
      beforeEach(() => {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true,
          rectLimits: limits,
          style: { ...defaultStyle,
            width: '180px',
            left: '20px'
          }
        });
      });
      it('should not change transition beyond given rect', () => {
        const {
          drag,
          getByTestId
        } = utils;
        const targetElement = getByTestId('main');
        const rect = targetElement.getBoundingClientRect();
        const startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        const delta = {
          x: -50,
          y: -90
        };
        drag({
          start: startAt,
          delta
        });
        (0, _chai.expect)(targetElement.getBoundingClientRect()).to.include({
          left: limits.left,
          top: limits.top
        });
      });
      it('should keep limits when dragging more than once', () => {
        const {
          drag,
          getByTestId
        } = utils;
        const targetElement = getByTestId('main');
        targetElement.style.right = '50px';
        targetElement.style.left = 'auto';
        const rect = targetElement.getBoundingClientRect();
        const startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        const delta = {
          x: 15,
          y: 1
        };
        drag({
          start: startAt,
          delta
        });
        drag({
          start: {
            clientX: startAt.clientX + delta.x,
            clientY: startAt.clientY + delta.y
          },
          delta: {
            x: 50,
            y: 0
          }
        });
        const {
          left,
          width
        } = targetElement.getBoundingClientRect();
        (0, _chai.expect)(left).to.equal(limits.right - width);
      });
    });
  });

  function Consumer(props) {
    const {
      targetRef,
      handleRef,
      getTargetProps,
      delta,
      dragging
    } = (0, _Draggable.useDraggable)(props);
    const {
      style = defaultStyle
    } = props;
    return _react.default.createElement("main", _extends({
      className: "container",
      ref: targetRef,
      "data-testid": "main",
      style: style
    }, getTargetProps()), dragging && _react.default.createElement("span", null, "Dragging to:"), _react.default.createElement("output", null, delta.x, ", ", delta.y), _react.default.createElement("button", {
      className: "handle",
      ref: handleRef
    }, "handle"));
  }

  function setup(props = {}) {
    const getters = (0, _react2.render)(_react.default.createElement(Consumer, props));

    function drag({
      start = {
        clientX: 0,
        clientY: 0
      },
      delta = {
        x: 0,
        y: 0
      },
      touch = false
    } = {}) {
      beginDrag(start, touch);
      move({
        clientX: start.clientX + delta.x,
        clientY: start.clientY + delta.y
      }, touch);
      endDrag({
        clientX: start.clientX + delta.x,
        clientY: start.clientY + delta.y
      }, touch);
    }

    function beginDrag(start, touch = false) {
      const target = getters.getByText(/handle/);

      if (touch) {
        _react2.fireEvent.touchStart(target, {
          touches: [createTouch({
            target,
            ...start
          })]
        });
      } else {
        _react2.fireEvent.mouseDown(target, start);
      }
    }

    function move(to, touch) {
      const target = getters.getByText(/handle/);

      if (touch) {
        _react2.fireEvent.touchMove(target, {
          touches: [createTouch({
            target,
            ...to
          })]
        });
      } else {
        _react2.fireEvent.mouseMove(target, to);
      }
    }

    function endDrag(end, touch) {
      const target = getters.getByText(/handle/);

      if (touch) {
        _react2.fireEvent.touchEnd(getters.getByText(/handle/), {
          changedTouches: [createTouch({
            target,
            ...end
          })]
        });
      } else {
        _react2.fireEvent.mouseUp(getters.getByText(/handle/), end);
      }
    }

    return { ...getters,
      beginDrag,
      move,
      drag
    };
  }
});

function createTouch({
  target,
  ...rest
}) {
  return new Touch({
    identifier: Date.now(),
    target,
    ...rest
  });
}